# https://dashscope.console.aliyun.com/model
import json
import aiohttp

from logging import getLogger
logger = getLogger(__name__)

class CAITongYiQianWen():
    API_URL = "https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation"
    def __init__(self, api_key : str,
                 _model : str = "qwen-1.8b-chat",
                 *args, **kwargs) -> None:
        self.api_key = api_key 
        self._modual_name = kwargs.get("model", _model)
        self.use_stream_api=kwargs.get("use_stream_api", False)

    def _transform_raw(self, response) -> str:
        return response["output"]["choices"][0]["message"]["content"]

    def transform_chunks(self, response: str, *args):
        current_msg = ''
        for line in response.splitlines():
            if line.startswith("data:"):
                current_msg += line[5:].strip()
            elif len(line) == 0:
                if current_msg:
                    return self._transform_raw(json.loads(current_msg))

    async def post_prompt(self, messages: dict = {}, is_stream: bool = False, header: dict[str, str] = {}):
        try:
            async with aiohttp.ClientSession() as client:
                async with client.post(self.API_URL,
                                         headers={
                                           'Content-Type': 'application/json',
                                           'Authorization': 'Bearer ' + self.api_key,
                                           **header,
                                           None if not is_stream else 'X-DashScope-SSE': 'enable'
                                         },
                                         json=messages) as response:
                    if is_stream:
                        async for chunk in response.content.iter_any():
                            yield chunk.decode('utf-8')
                    else:
                        data = await response.read()
                        yield data.decode('utf-8')
        except Exception as e:
            logger.exception("post_prompt")
            raise e

    async def execute_stream(self, data):
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": True
                }
            }, is_stream = True)
        return response
    
    async def execute_nonstream(self, data):
        response = self.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": False
                }
            })
        # ret = response.json()
        # return self._transform_raw(ret)

    def prepare_input(self, data):
        return [{'role': 'user', 'content': f'{data}'}]